@@ -35,6 +35,8 @@ extern int optreset;		/* defined by BSD, but not others */
 #include "debugfs.h"
 #include "uuid/uuid.h"
 
+#include "../version.h"
+
 extern ss_request_table debug_cmds;
 
 ext2_filsys current_fs = NULL;
@@ -74,7 +76,7 @@ errout:
 void do_open_filesys(int argc, char **argv)
 {
 	const char	*usage = "Usage: open [-w] <device>";
-	char	c;
+	int	c;
 	int open_flags = 0;
 	
 	optind = 0;
@@ -248,7 +250,7 @@ void do_show_super_stats(int argc, char *argv[])
 	close_pager(out);
 }
 
-void do_dirty_filesys(int argc, char *argv[])
+void do_dirty_filesys(int argc, char **argv)
 {
 	if (check_fs_open(argv[0]))
 		return;
@@ -306,16 +308,24 @@ static void dump_inode(ino_t inode_num, struct ext2_inode inode)
 	fprintf(out, "Inode: %ld   Type: %s    ", inode_num, i_type);
 	fprintf(out, "Mode:  %04o   Flags: 0x%x   Version: %d\n",
 		inode.i_mode & 0777, inode.i_flags, inode.i_version);
-	fprintf(out, "User: %5d   Group: %5d   Size: %d\n",  
-		inode.i_uid, inode.i_gid, inode.i_size);
+	fprintf(out, "User: %5d   Group: %5d   Size: ",
+		inode.i_uid, inode.i_gid);
+	if (LINUX_S_ISDIR(inode.i_mode))
+		fprintf(out, "%d\n", inode.i_size);
+	else {
+		__u64 i_size = (inode.i_size |
+				((unsigned long long)inode.i_size_high << 32));
+		
+		fprintf(out, "%lld\n", i_size);
+	}
 	if (current_fs->super->s_creator_os == EXT2_OS_HURD)
 		fprintf(out,
 			"File ACL: %d    Directory ACL: %d Translator: %d\n",
-			inode.i_file_acl, inode.i_dir_acl,
+			inode.i_file_acl, LINUX_S_ISDIR(inode.i_mode) ? inode.i_dir_acl : 0,
 			inode.osd1.hurd1.h_i_translator);
 	else
 		fprintf(out, "File ACL: %d    Directory ACL: %d\n",
-			inode.i_file_acl, inode.i_dir_acl);
+			inode.i_file_acl, LINUX_S_ISDIR(inode.i_mode) ? inode.i_dir_acl : 0);
 	fprintf(out, "Links: %d   Blockcount: %d\n", inode.i_links_count,
 		inode.i_blocks);
 	switch (os) {
@@ -681,7 +691,10 @@ void do_modify_inode(int argc, char *argv[])
 	modify_u32(argv[0], "Reserved1", decimal_format, &inode.i_reserved1);
 #endif
 	modify_u32(argv[0], "File acl", decimal_format, &inode.i_file_acl);
-	modify_u32(argv[0], "Directory acl", decimal_format, &inode.i_dir_acl);
+	if (LINUX_S_ISDIR(inode.i_mode))
+		modify_u32(argv[0], "Directory acl", decimal_format, &inode.i_dir_acl);
+	else
+		modify_u32(argv[0], "High 32bits of size", decimal_format, &inode.i_size_high);
 
 	if (current_fs->super->s_creator_os == EXT2_OS_HURD)
 		modify_u32(argv[0], "Translator Block",
@@ -946,119 +959,48 @@ void do_find_free_inode(int argc, char *argv[])
 		printf("Free inode found: %ld\n", free_inode);
 }
 
-struct copy_file_struct {
-	unsigned long size;
-	int	done, fd, blocks;
-	errcode_t err;
-};
-
-static int copy_file_proc(ext2_filsys to_fs,
-			   blk_t	*blocknr,
-			   int	blockcnt,
-			   void	*private)
-{
-	struct copy_file_struct *cs = (struct copy_file_struct *) private;
-	blk_t	new_blk;
-	static blk_t	last_blk = 0;
-	char		*block;
-	errcode_t	retval;
-	int		group;
-	int		nr;
-	
-	if (*blocknr) {
-		new_blk = *blocknr;
-	} else {
-		retval = ext2fs_new_block(to_fs, last_blk, 0, &new_blk);
-		if (retval) {
-			cs->err = retval;
-			return BLOCK_ABORT;
-		}
-	}
-	last_blk = new_blk;
-	block = malloc(to_fs->blocksize);
-	if (!block) {
-		cs->err = ENOMEM;
-		return BLOCK_ABORT;
-	}
-	if (blockcnt >= 0) {
-		nr = read(cs->fd, block, to_fs->blocksize);
-	} else {
-		nr = to_fs->blocksize;
-		memset(block, 0, nr);
-	}
-	if (nr == 0) {
-		cs->done = 1;
-		return BLOCK_ABORT;
-	}
-	if (nr < 0) {
-		cs->err = nr;
-		return BLOCK_ABORT;
-	}
-	retval = io_channel_write_blk(to_fs->io, new_blk, 1, block);
-	if (retval) {
-		cs->err = retval;
-		return BLOCK_ABORT;
-	}
-	free(block);
-	if (blockcnt >= 0)
-		cs->size += nr;
-	cs->blocks += to_fs->blocksize / 512;
-	printf("%ld(%d) ", cs->size, blockcnt);
-	fflush(stdout);
-	if (nr < to_fs->blocksize) {
-		cs->done = 1;
-		printf("\n");
-	}
-	*blocknr = new_blk;
-	ext2fs_mark_block_bitmap(to_fs->block_map, new_blk);
-	ext2fs_mark_bb_dirty(to_fs);
-	group = ext2fs_group_of_blk(to_fs, new_blk);
-	to_fs->group_desc[group].bg_free_blocks_count--;
-	to_fs->super->s_free_blocks_count--;
-	ext2fs_mark_super_dirty(to_fs);
-	if (cs->done)
-		return (BLOCK_CHANGED | BLOCK_ABORT);
-	else
-		return BLOCK_CHANGED;
-}
-
 static errcode_t copy_file(int fd, ino_t newfile)
 {
+	ext2_file_t	e2_file;
 	errcode_t	retval;
-	struct	copy_file_struct cs;
-	struct ext2_inode	inode;
+	unsigned int	got, written;
+	char		buf[8192];
+	char		*ptr;
 
-	cs.fd = fd;
-	cs.done = 0;
-	cs.err = 0;
-	cs.size = 0;
-	cs.blocks = 0;
-	
-	retval = ext2fs_block_iterate(current_fs, newfile,
-				      BLOCK_FLAG_APPEND,
-				      0, copy_file_proc, &cs);
-
-	if (cs.err)
-		return cs.err;
-	if (!cs.done)
-		return EXT2_ET_EXPAND_DIR_ERR;
-
-	/*
-	 * Update the size and block count fields in the inode.
-	 */
-	retval = ext2fs_read_inode(current_fs, newfile, &inode);
-	if (retval)
-		return retval;
-	
-	inode.i_blocks += cs.blocks;
-
-	retval = ext2fs_write_inode(current_fs, newfile, &inode);
+	retval = ext2fs_file_open(current_fs, newfile,
+				  EXT2_FILE_WRITE, &e2_file);
 	if (retval)
 		return retval;
 
-	return 0;
+	while (1) {
+		got = read(fd, buf, sizeof(buf));
+		if (got == 0)
+			break;
+		if (got < 0) {
+			retval = errno;
+			goto fail;
+		}
+		ptr = buf;
+		while (got > 0) {
+			retval = ext2fs_file_write(e2_file, ptr,
+						   got, &written);
+			if (retval)
+				goto fail;
+
+			got -= written;
+			ptr += written;
+		}
+	}
+	retval = ext2fs_file_close(e2_file);
+
+	return retval;
+
+fail:
+	(void) ext2fs_file_close(e2_file);
+	return retval;
 }
 
+
 void do_write(int argc, char *argv[])
 {
 	int	fd;
@@ -1066,6 +1008,7 @@ void do_write(int argc, char *argv[])
 	ino_t	newfile;
 	errcode_t retval;
 	struct ext2_inode inode;
+	dgrp_t group;
 
 	if (check_fs_open(argv[0]))
 		return;
@@ -1092,6 +1035,10 @@ void do_write(int argc, char *argv[])
 		close(fd);
 		return;
 	}
+	group = ext2fs_group_of_ino(current_fs, newfile);
+	current_fs->group_desc[group].bg_free_inodes_count--;
+	current_fs->super->s_free_inodes_count--;
+	ext2fs_mark_super_dirty(current_fs);
 	printf("Allocated inode: %ld\n", newfile);
 	retval = ext2fs_link(current_fs, cwd, argv[2], newfile, 0);
 	if (retval) {
@@ -1404,20 +1351,23 @@ static int source_file(const char *cmd_file, int sci_idx)
 	return exit_status;
 }
 
-void main(int argc, char **argv)
+int main(int argc, char **argv)
 {
 	int		retval;
 	int		sci_idx;
-	const char	*usage = "Usage: debugfs [[-w] device]";
-	char		c;
+	const char	*usage = "Usage: debugfs [-f cmd_file] [-R request] [-V] [[-w] device]";
+	int		c;
 	int		open_flags = 0;
 	char		*request = 0;
 	int		exit_status = 0;
 	char		*cmd_file = 0;
 	
 	initialize_ext2_error_table();
+	fprintf (stderr, "debugfs %s, %s for EXT2 FS %s, %s\n",
+		 E2FSPROGS_VERSION, E2FSPROGS_DATE,
+		 EXT2FS_VERSION, EXT2FS_DATE);
 
-	while ((c = getopt (argc, argv, "wR:f:")) != EOF) {
+	while ((c = getopt (argc, argv, "wR:f:V")) != EOF) {
 		switch (c) {
 		case 'R':
 			request = optarg;
@@ -1428,9 +1378,14 @@ void main(int argc, char **argv)
 		case 'w':
 			open_flags = EXT2_FLAG_RW;
 			break;
+		case 'V':
+			/* Print version number and exit */
+			fprintf(stderr, "\tUsing %s\n",
+				error_message(EXT2_ET_BASE));
+			exit(0);
 		default:
 			com_err(argv[0], 0, usage);
-			return;
+			return 1;
 		}
 	}
 	if (optind < argc)
