@@ -14,7 +14,7 @@
 #include "e2fsck.h"
 
 static int check_bb_inode_blocks(ext2_filsys fs, blk_t *block_nr, int blockcnt,
-				 void *private);
+				 void *priv_data);
 
 
 static void invalid_block(ext2_filsys fs, blk_t blk)
@@ -23,15 +23,16 @@ static void invalid_block(ext2_filsys fs, blk_t blk)
 	return;
 }
 
-void read_bad_blocks_file(ext2_filsys fs, const char *bad_blocks_file,
+void read_bad_blocks_file(e2fsck_t ctx, const char *bad_blocks_file,
 			  int replace_bad_blocks)
 {
+	ext2_filsys fs = ctx->fs;
 	errcode_t	retval;
 	badblocks_list	bb_list = 0;
 	FILE		*f;
 	char		buf[1024];
 
-	read_bitmaps(fs);
+	e2fsck_read_bitmaps(ctx);
 
 	/*
 	 * Make sure the bad block inode is sane.  If there are any
@@ -42,7 +43,7 @@ void read_bad_blocks_file(ext2_filsys fs, const char *bad_blocks_file,
 	if (retval) {
 		com_err("ext2fs_block_iterate", retval,
 			"while sanity checking the bad blocks inode");
-		fatal_error(0);
+		goto fatal;
 	}
 	
 	/*
@@ -54,7 +55,7 @@ void read_bad_blocks_file(ext2_filsys fs, const char *bad_blocks_file,
 		if (retval) {
 			com_err("ext2fs_read_bb_inode", retval,
 				"while reading the bad blocks inode");
-			fatal_error(0);
+			goto fatal;
 		}
 	}
 	
@@ -68,17 +69,17 @@ void read_bad_blocks_file(ext2_filsys fs, const char *bad_blocks_file,
 		if (!f) {
 			com_err("read_bad_blocks_file", errno,
 				"while trying to open %s", bad_blocks_file);
-			fatal_error(0);
+			goto fatal;
 		}
 	} else {
-		sprintf(buf, "badblocks %s%s %d", preen ? "" : "-s ",
-			fs->device_name,
-			fs->super->s_blocks_count);
+		sprintf(buf, "badblocks -b %d %s%s %d", fs->blocksize,
+			(ctx->options & E2F_OPT_PREEN) ? "" : "-s ",
+			fs->device_name, fs->super->s_blocks_count);
 		f = popen(buf, "r");
 		if (!f) {
 			com_err("read_bad_blocks_file", errno,
 				"while trying popen '%s'", buf);
-			fatal_error(0);
+			goto fatal;
 		}
 	}
 	retval = ext2fs_read_bb_FILE(fs, f, &bb_list, invalid_block);
@@ -89,7 +90,7 @@ void read_bad_blocks_file(ext2_filsys fs, const char *bad_blocks_file,
 	if (retval) {
 		com_err("ext2fs_read_bb_FILE", retval,
 			"while reading in list of bad blocks from file");
-		fatal_error(0);
+		goto fatal;
 	}
 	
 	/*
@@ -99,20 +100,25 @@ void read_bad_blocks_file(ext2_filsys fs, const char *bad_blocks_file,
 	if (retval) {
 		com_err("ext2fs_update_bb_inode", retval,
 			"while updating bad block inode");
-		fatal_error(0);
+		goto fatal;
 	}
 
 	badblocks_list_free(bb_list);
 	return;
+	
+fatal:
+	ctx->flags |= E2F_FLAG_ABORT;
+	return;
+	
 }
 
-void test_disk(ext2_filsys fs)
+void test_disk(e2fsck_t ctx)
 {
-	read_bad_blocks_file(fs, 0, 1);
+	read_bad_blocks_file(ctx, 0, 0);
 }
 
 static int check_bb_inode_blocks(ext2_filsys fs, blk_t *block_nr, int blockcnt,
-				 void *private)
+				 void *priv_data)
 {
 	if (!*block_nr)
 		return 0;
