@@ -1,6 +1,6 @@
 /*
- * $Header: /mit/krb5/.cvsroot/src/util/et/error_message.c,v 5.0 1993/04/13 19:56:17 tytso Exp $
- * $Source: /mit/krb5/.cvsroot/src/util/et/error_message.c,v $
+ * $Header: /mit/tytso/linux/CVS-REPO/e2fsprogs/lib/et/error_message.c,v 1.14 1997/08/10 23:00:57 tytso Exp $
+ * $Source: /mit/tytso/linux/CVS-REPO/e2fsprogs/lib/et/error_message.c,v $
  * $Locker:  $
  *
  * Copyright 1987 by the Student Information Processing Board
@@ -11,35 +11,45 @@
 
 #include <stdio.h>
 #include <string.h>
+#include <errno.h>
+#include "com_err.h"
 #include "error_table.h"
 #include "mit-sipb-copyright.h"
 #include "internal.h"
 
-static const char rcsid[] =
-    "$Header: /mit/krb5/.cvsroot/src/util/et/error_message.c,v 5.0 1993/04/13 19:56:17 tytso Exp $";
-static const char copyright[] =
-    "Copyright 1986, 1987, 1988 by the Student Information Processing Board\nand the department of Information Systems\nof the Massachusetts Institute of Technology";
-
 static char buffer[25];
 
 struct et_list * _et_list = (struct et_list *) NULL;
 
+
+#ifdef __STDC__
+const char * error_message (errcode_t code)
+#else
 const char * error_message (code)
-long	code;
+	errcode_t	code;
+#endif
 {
     int offset;
     struct et_list *et;
-    int table_num;
+    errcode_t table_num;
     int started = 0;
     char *cp;
 
-    offset = code & ((1<<ERRCODE_RANGE)-1);
+    offset = (int) (code & ((1<<ERRCODE_RANGE)-1));
     table_num = code - offset;
     if (!table_num) {
+#ifdef HAS_SYS_ERRLIST
 	if (offset < sys_nerr)
 	    return(sys_errlist[offset]);
 	else
 	    goto oops;
+#else
+	cp = strerror(offset);
+	if (cp)
+	    return(cp);
+	else
+	    goto oops;
+#endif
     }
     for (et = _et_list; et; et = et->next) {
 	if (et->table->base == table_num) {
