@@ -11,16 +11,23 @@
  */
 
 #include <stdio.h>
+#if HAVE_UNISTD_H
 #include <unistd.h>
-#include <stdlib.h>
+#endif
 #include <time.h>
+#include <string.h>
+#if HAVE_SYS_STAT_H
 #include <sys/stat.h>
+#endif
+#if HAVE_SYS_TYPES_H
 #include <sys/types.h>
-#if HAVE_ERRNO_H
-#include <errno.h>
 #endif
 
+#if EXT2_FLAT_INCLUDES
+#include "ext2_fs.h"
+#else
 #include <linux/ext2_fs.h>
+#endif
 
 #include "ext2fs.h"
 
@@ -33,7 +40,7 @@
 errcode_t ext2fs_new_inode(ext2_filsys fs, ino_t dir, int mode,
 			   ext2fs_inode_bitmap map, ino_t *ret)
 {
-	int	dir_group = 0;
+	ino_t	dir_group = 0;
 	ino_t	i;
 	ino_t	start_inode;
 
@@ -61,7 +68,7 @@ errcode_t ext2fs_new_inode(ext2_filsys fs, ino_t dir, int mode,
 	} while (i != start_inode);
 	
 	if (ext2fs_test_inode_bitmap(map, i))
-		return ENOSPC;
+		return EXT2_ET_INODE_ALLOC_FAIL;
 	*ret = i;
 	return 0;
 }
@@ -93,7 +100,56 @@ errcode_t ext2fs_new_block(ext2_filsys fs, blk_t goal,
 		if (i >= fs->super->s_blocks_count)
 			i = fs->super->s_first_data_block;
 	} while (i != goal);
-	return ENOSPC;
+	return EXT2_ET_BLOCK_ALLOC_FAIL;
+}
+
+/*
+ * This function zeros out the allocated block, and updates all of the
+ * appropriate filesystem records.
+ */
+errcode_t ext2fs_alloc_block(ext2_filsys fs, blk_t goal,
+			     char *block_buf, blk_t *ret)
+{
+	errcode_t	retval;
+	blk_t		block;
+	int		group;
+	char		*buf = 0;
+
+	if (!block_buf) {
+		retval = ext2fs_get_mem(fs->blocksize, (void **) &buf);
+		if (retval)
+			return retval;
+		block_buf = buf;
+	}
+	memset(block_buf, 0, fs->blocksize);
+
+	if (!fs->block_map) {
+		retval = ext2fs_read_block_bitmap(fs);
+		if (retval)
+			goto fail;
+	}
+
+	retval = ext2fs_new_block(fs, goal, 0, &block);
+	if (retval)
+		goto fail;
+
+	retval = io_channel_write_blk(fs->io, block, 1, block_buf);
+	if (retval)
+		goto fail;
+	
+	fs->super->s_free_blocks_count--;
+	group = ext2fs_group_of_blk(fs, block);
+	fs->group_desc[group].bg_free_blocks_count--;
+	ext2fs_mark_block_bitmap(fs->block_map, block);
+	ext2fs_mark_super_dirty(fs);
+	ext2fs_mark_bb_dirty(fs);
+	*ret = block;
+	return 0;
+
+fail:
+	if (buf)
+		ext2fs_free_mem((void **) &buf);
+	return retval;
 }
 
 errcode_t ext2fs_get_free_blocks(ext2_filsys fs, blk_t start, blk_t finish,
@@ -122,6 +178,6 @@ errcode_t ext2fs_get_free_blocks(ext2_filsys fs, blk_t start, blk_t finish,
 		}
 		b++;
 	} while (b != finish);
-	return ENOSPC;
+	return EXT2_ET_BLOCK_ALLOC_FAIL;
 }
 
