@@ -11,13 +11,15 @@
 
 #include <stdio.h>
 #include <string.h>
+#if HAVE_UNISTD_H
 #include <unistd.h>
-#include <stdlib.h>
-#if HAVE_ERRNO_H
-#include <errno.h>
 #endif
 
+#if EXT2_FLAT_INCLUDES
+#include "ext2_fs.h"
+#else
 #include <linux/ext2_fs.h>
+#endif
 
 #include "ext2fs.h"
 
@@ -25,18 +27,18 @@ struct block_context {
 	ext2_filsys	fs;
 	int (*func)(ext2_filsys	fs,
 		    blk_t	*blocknr,
-		    int		bcount,
+		    e2_blkcnt_t	bcount,
 		    blk_t	ref_blk,
 		    int		ref_offset,
-		    void	*private);
-	int		bcount;
+		    void	*priv_data);
+	e2_blkcnt_t	bcount;
 	int		bsize;
 	int		flags;
 	errcode_t	errcode;
 	char	*ind_buf;
 	char	*dind_buf;
 	char	*tind_buf;
-	void	*private;
+	void	*priv_data;
 };
 
 static int block_iterate_ind(blk_t *ind_block, blk_t ref_block,
@@ -46,13 +48,16 @@ static int block_iterate_ind(blk_t *ind_block, blk_t ref_block,
 	int	i, flags, limit, offset;
 	blk_t	*block_nr;
 
+	limit = ctx->fs->blocksize >> 2;
 	if (!(ctx->flags & BLOCK_FLAG_DEPTH_TRAVERSE) &&
 	    !(ctx->flags & BLOCK_FLAG_DATA_ONLY))
 		ret = (*ctx->func)(ctx->fs, ind_block,
 				   BLOCK_COUNT_IND, ref_block,
-				   ref_offset, ctx->private);
-	if (!*ind_block || (ret & BLOCK_ABORT))
+				   ref_offset, ctx->priv_data);
+	if (!*ind_block || (ret & BLOCK_ABORT)) {
+		ctx->bcount += limit;
 		return ret;
+	}
 	if (*ind_block >= ctx->fs->super->s_blocks_count ||
 	    *ind_block < ctx->fs->super->s_first_data_block) {
 		ctx->errcode = EXT2_ET_BAD_IND_BLOCK;
@@ -65,7 +70,6 @@ static int block_iterate_ind(blk_t *ind_block, blk_t ref_block,
 		ret |= BLOCK_ERROR;
 		return ret;
 	}
-	limit = ctx->fs->blocksize >> 2;
 	if ((ctx->fs->flags & EXT2_FLAG_SWAP_BYTES) ||
 	    (ctx->fs->flags & EXT2_FLAG_SWAP_BYTES_READ)) {
 		block_nr = (blk_t *) ctx->ind_buf;
@@ -78,7 +82,7 @@ static int block_iterate_ind(blk_t *ind_block, blk_t ref_block,
 		for (i = 0; i < limit; i++, ctx->bcount++, block_nr++) {
 			flags = (*ctx->func)(ctx->fs, block_nr, ctx->bcount,
 					     *ind_block, offset, 
-					     ctx->private);
+					     ctx->priv_data);
 			changed	|= flags;
 			if (flags & BLOCK_ABORT) {
 				ret |= BLOCK_ABORT;
@@ -92,7 +96,7 @@ static int block_iterate_ind(blk_t *ind_block, blk_t ref_block,
 				continue;
 			flags = (*ctx->func)(ctx->fs, block_nr, ctx->bcount,
 					     *ind_block, offset, 
-					     ctx->private);
+					     ctx->priv_data);
 			changed	|= flags;
 			if (flags & BLOCK_ABORT) {
 				ret |= BLOCK_ABORT;
@@ -118,7 +122,7 @@ static int block_iterate_ind(blk_t *ind_block, blk_t ref_block,
 	    !(ret & BLOCK_ABORT))
 		ret |= (*ctx->func)(ctx->fs, ind_block,
 				    BLOCK_COUNT_IND, ref_block,
-				    ref_offset, ctx->private);
+				    ref_offset, ctx->priv_data);
 	return ret;
 }
 	
@@ -129,13 +133,16 @@ static int block_iterate_dind(blk_t *dind_block, blk_t ref_block,
 	int	i, flags, limit, offset;
 	blk_t	*block_nr;
 
+	limit = ctx->fs->blocksize >> 2;
 	if (!(ctx->flags & BLOCK_FLAG_DEPTH_TRAVERSE) &&
 	    !(ctx->flags & BLOCK_FLAG_DATA_ONLY))
 		ret = (*ctx->func)(ctx->fs, dind_block,
 				   BLOCK_COUNT_DIND, ref_block,
-				   ref_offset, ctx->private);
-	if (!*dind_block || (ret & BLOCK_ABORT))
+				   ref_offset, ctx->priv_data);
+	if (!*dind_block || (ret & BLOCK_ABORT)) {
+		ctx->bcount += limit*limit;
 		return ret;
+	}
 	if (*dind_block >= ctx->fs->super->s_blocks_count ||
 	    *dind_block < ctx->fs->super->s_first_data_block) {
 		ctx->errcode = EXT2_ET_BAD_DIND_BLOCK;
@@ -148,7 +155,6 @@ static int block_iterate_dind(blk_t *dind_block, blk_t ref_block,
 		ret |= BLOCK_ERROR;
 		return ret;
 	}
-	limit = ctx->fs->blocksize >> 2;
 	if ((ctx->fs->flags & EXT2_FLAG_SWAP_BYTES) ||
 	    (ctx->fs->flags & EXT2_FLAG_SWAP_BYTES_READ)) {
 		block_nr = (blk_t *) ctx->dind_buf;
@@ -171,8 +177,10 @@ static int block_iterate_dind(blk_t *dind_block, blk_t ref_block,
 		}
 	} else {
 		for (i = 0; i < limit; i++, block_nr++) {
-			if (*block_nr == 0)
+			if (*block_nr == 0) {
+				ctx->bcount += limit;
 				continue;
+			}
 			flags = block_iterate_ind(block_nr,
 						  *dind_block, offset,
 						  ctx);
@@ -201,7 +209,7 @@ static int block_iterate_dind(blk_t *dind_block, blk_t ref_block,
 	    !(ret & BLOCK_ABORT))
 		ret |= (*ctx->func)(ctx->fs, dind_block,
 				    BLOCK_COUNT_DIND, ref_block,
-				    ref_offset, ctx->private);
+				    ref_offset, ctx->priv_data);
 	return ret;
 }
 	
@@ -212,13 +220,16 @@ static int block_iterate_tind(blk_t *tind_block, blk_t ref_block,
 	int	i, flags, limit, offset;
 	blk_t	*block_nr;
 
+	limit = ctx->fs->blocksize >> 2;
 	if (!(ctx->flags & BLOCK_FLAG_DEPTH_TRAVERSE) &&
 	    !(ctx->flags & BLOCK_FLAG_DATA_ONLY))
 		ret = (*ctx->func)(ctx->fs, tind_block,
 				   BLOCK_COUNT_TIND, ref_block,
-				   ref_offset, ctx->private);
-	if (!*tind_block || (ret & BLOCK_ABORT))
+				   ref_offset, ctx->priv_data);
+	if (!*tind_block || (ret & BLOCK_ABORT)) {
+		ctx->bcount += limit*limit*limit;
 		return ret;
+	}
 	if (*tind_block >= ctx->fs->super->s_blocks_count ||
 	    *tind_block < ctx->fs->super->s_first_data_block) {
 		ctx->errcode = EXT2_ET_BAD_TIND_BLOCK;
@@ -231,7 +242,6 @@ static int block_iterate_tind(blk_t *tind_block, blk_t ref_block,
 		ret |= BLOCK_ERROR;
 		return ret;
 	}
-	limit = ctx->fs->blocksize >> 2;
 	if ((ctx->fs->flags & EXT2_FLAG_SWAP_BYTES) ||
 	    (ctx->fs->flags & EXT2_FLAG_SWAP_BYTES_READ)) {
 		block_nr = (blk_t *) ctx->tind_buf;
@@ -254,8 +264,10 @@ static int block_iterate_tind(blk_t *tind_block, blk_t ref_block,
 		}
 	} else {
 		for (i = 0; i < limit; i++, block_nr++) {
-			if (*block_nr == 0)
+			if (*block_nr == 0) {
+				ctx->bcount += limit*limit;
 				continue;
+			}
 			flags = block_iterate_dind(block_nr,
 						   *tind_block,
 						   offset, ctx);
@@ -284,7 +296,7 @@ static int block_iterate_tind(blk_t *tind_block, blk_t ref_block,
 	    !(ret & BLOCK_ABORT))
 		ret |= (*ctx->func)(ctx->fs, tind_block,
 				    BLOCK_COUNT_TIND, ref_block,
-				    ref_offset, ctx->private);
+				    ref_offset, ctx->priv_data);
 	
 	return ret;
 }
@@ -295,11 +307,11 @@ errcode_t ext2fs_block_iterate2(ext2_filsys fs,
 				char *block_buf,
 				int (*func)(ext2_filsys fs,
 					    blk_t	*blocknr,
-					    int	blockcnt,
+					    e2_blkcnt_t	blockcnt,
 					    blk_t	ref_blk,
 					    int		ref_offset,
-					    void	*private),
-				void *private)
+					    void	*priv_data),
+				void *priv_data)
 {
 	int	i;
 	int	got_inode = 0;
@@ -308,24 +320,41 @@ errcode_t ext2fs_block_iterate2(ext2_filsys fs,
 	struct ext2_inode inode;
 	errcode_t	retval;
 	struct block_context ctx;
+	int	limit;
 
 	EXT2_CHECK_MAGIC(fs, EXT2_ET_MAGIC_EXT2FS_FILSYS);
 
-	ret = ext2fs_get_blocks(fs, ino, blocks);
-	if (ret)
-		return ret;
+	/*
+	 * Check to see if we need to limit large files
+	 */
+	if (flags & BLOCK_FLAG_NO_LARGE) {
+		ctx.errcode = ext2fs_read_inode(fs, ino, &inode);
+		if (ctx.errcode)
+			goto abort;
+		got_inode = 1;
+		if (!LINUX_S_ISDIR(inode.i_mode) &&
+		    (inode.i_size_high != 0))
+			return EXT2_ET_FILE_TOO_BIG;
+	}
+
+	retval = ext2fs_get_blocks(fs, ino, blocks);
+	if (retval)
+		return retval;
+
+	limit = fs->blocksize >> 2;
 
 	ctx.fs = fs;
 	ctx.func = func;
-	ctx.private = private;
+	ctx.priv_data = priv_data;
 	ctx.flags = flags;
 	ctx.bcount = 0;
 	if (block_buf) {
 		ctx.ind_buf = block_buf;
 	} else {
-		ctx.ind_buf = malloc(fs->blocksize * 3);
-		if (!ctx.ind_buf)
-			return ENOMEM;
+		retval = ext2fs_get_mem(fs->blocksize * 3,
+					(void **) &ctx.ind_buf);
+		if (retval)
+			return retval;
 	}
 	ctx.dind_buf = ctx.ind_buf + fs->blocksize;
 	ctx.tind_buf = ctx.dind_buf + fs->blocksize;
@@ -343,7 +372,7 @@ errcode_t ext2fs_block_iterate2(ext2_filsys fs,
 			ret |= (*ctx.func)(fs,
 					   &inode.osd1.hurd1.h_i_translator,
 					   BLOCK_COUNT_TRANSLATOR,
-					   0, 0, private);
+					   0, 0, priv_data);
 			if (ret & BLOCK_ABORT)
 				goto abort;
 		}
@@ -355,26 +384,28 @@ errcode_t ext2fs_block_iterate2(ext2_filsys fs,
 	for (i = 0; i < EXT2_NDIR_BLOCKS ; i++, ctx.bcount++) {
 		if (blocks[i] || (flags & BLOCK_FLAG_APPEND)) {
 			ret |= (*ctx.func)(fs, &blocks[i],
-					    ctx.bcount, 0, 0, private);
+					    ctx.bcount, 0, i, priv_data);
 			if (ret & BLOCK_ABORT)
 				goto abort;
 		}
 	}
 	if (*(blocks + EXT2_IND_BLOCK) || (flags & BLOCK_FLAG_APPEND)) {
 		ret |= block_iterate_ind(blocks + EXT2_IND_BLOCK,
-					 0, 0, &ctx);
+					 0, EXT2_IND_BLOCK, &ctx);
 		if (ret & BLOCK_ABORT)
 			goto abort;
-	}
+	} else
+		ctx.bcount += limit;
 	if (*(blocks + EXT2_DIND_BLOCK) || (flags & BLOCK_FLAG_APPEND)) {
 		ret |= block_iterate_dind(blocks + EXT2_DIND_BLOCK,
-					  0, 0, &ctx);
+					  0, EXT2_DIND_BLOCK, &ctx);
 		if (ret & BLOCK_ABORT)
 			goto abort;
-	}
+	} else
+		ctx.bcount += limit * limit;
 	if (*(blocks + EXT2_TIND_BLOCK) || (flags & BLOCK_FLAG_APPEND)) {
 		ret |= block_iterate_tind(blocks + EXT2_TIND_BLOCK,
-					  0, 0, &ctx);
+					  0, EXT2_TIND_BLOCK, &ctx);
 		if (ret & BLOCK_ABORT)
 			goto abort;
 	}
@@ -394,25 +425,32 @@ abort:
 	}
 
 	if (!block_buf)
-		free(ctx.ind_buf);
+		ext2fs_free_mem((void **) &ctx.ind_buf);
 
 	return (ret & BLOCK_ERROR) ? ctx.errcode : 0;
 }
 
+/*
+ * Emulate the old ext2fs_block_iterate function!
+ */
+
 struct xlate {
 	int (*func)(ext2_filsys	fs,
 		    blk_t	*blocknr,
 		    int		bcount,
-		    void	*private);
+		    void	*priv_data);
 	void *real_private;
 };
 
-static int xlate_func(ext2_filsys fs, blk_t *blocknr, int blockcnt,
-		      blk_t ref_block, int ref_offset, void *private)
+#ifdef __TURBOC__
+#pragma argsused
+#endif
+static int xlate_func(ext2_filsys fs, blk_t *blocknr, e2_blkcnt_t blockcnt,
+		      blk_t ref_block, int ref_offset, void *priv_data)
 {
-	struct xlate *xl = private;
+	struct xlate *xl = (struct xlate *) priv_data;
 
-	return (*xl->func)(fs, blocknr, blockcnt, xl->real_private);
+	return (*xl->func)(fs, blocknr, (int) blockcnt, xl->real_private);
 }
 
 errcode_t ext2fs_block_iterate(ext2_filsys fs,
@@ -422,16 +460,15 @@ errcode_t ext2fs_block_iterate(ext2_filsys fs,
 			       int (*func)(ext2_filsys fs,
 					   blk_t	*blocknr,
 					   int	blockcnt,
-					   void	*private),
-			       void *private)
+					   void	*priv_data),
+			       void *priv_data)
 {
 	struct xlate xl;
 	
-	xl.real_private = private;
+	xl.real_private = priv_data;
 	xl.func = func;
 
-	return ext2fs_block_iterate2(fs, ino, flags, block_buf,
-				    xlate_func, &xl);
+	return ext2fs_block_iterate2(fs, ino, BLOCK_FLAG_NO_LARGE | flags,
+				     block_buf, xlate_func, &xl);
 }
 
-
