@@ -11,19 +11,21 @@
  */
 
 #include <stdio.h>
+#if HAVE_UNISTD_H
 #include <unistd.h>
-#include <stdlib.h>
+#endif
 #include <string.h>
 #include <time.h>
-#ifdef HAVE_ERRNO_H
-#include <errno.h>
-#endif
 
+#if EXT2_FLAT_INCLUDES
+#include "ext2_fs.h"
+#else
 #include <linux/ext2_fs.h>
+#endif
 
 #include "ext2fsP.h"
 
-static int dir_block_cmp(const void *a, const void *b);
+static EXT2_QSORT_TYPE dir_block_cmp(const void *a, const void *b);
 
 /*
  * Returns the number of directories in the filesystem as reported by
@@ -33,13 +35,20 @@ static int dir_block_cmp(const void *a, const void *b);
 errcode_t ext2fs_get_num_dirs(ext2_filsys fs, ino_t *ret_num_dirs)
 {
 	int	i;
-	ino_t	num_dirs;
+	ino_t	num_dirs, max_dirs;
 
 	EXT2_CHECK_MAGIC(fs, EXT2_ET_MAGIC_EXT2FS_FILSYS);
 	
 	num_dirs = 0;
-	for (i = 0; i < fs->group_desc_count; i++)
-		num_dirs += fs->group_desc[i].bg_used_dirs_count;
+	max_dirs = fs->super->s_inodes_per_group;
+	for (i = 0; i < fs->group_desc_count; i++) {
+		if (fs->group_desc[i].bg_used_dirs_count > max_dirs)
+			num_dirs += max_dirs / 8;
+		else
+			num_dirs += fs->group_desc[i].bg_used_dirs_count;
+	}
+	if (num_dirs > fs->super->s_inodes_count)
+		num_dirs = fs->super->s_inodes_count;
 
 	*ret_num_dirs = num_dirs;
 
@@ -64,9 +73,10 @@ static errcode_t make_dblist(ext2_filsys fs, ino_t size, ino_t count,
 	    (fs->dblist->magic == EXT2_ET_MAGIC_DBLIST))
 		return 0;
 
-	dblist = malloc(sizeof(struct ext2_struct_dblist));
-	if (!dblist)
-		return ENOMEM;
+	retval = ext2fs_get_mem(sizeof(struct ext2_struct_dblist),
+				(void **) &dblist);
+	if (retval)
+		return retval;
 	memset(dblist, 0, sizeof(struct ext2_struct_dblist));
 
 	dblist->magic = EXT2_ET_MAGIC_DBLIST;
@@ -77,14 +87,14 @@ static errcode_t make_dblist(ext2_filsys fs, ino_t size, ino_t count,
 		retval = ext2fs_get_num_dirs(fs, &dblist->size);
 		if (retval)
 			goto cleanup;
+		dblist->size = (dblist->size * 2) + 12;
 	}
-	len = sizeof(struct ext2_db_entry) * dblist->size;
+	len = (size_t) sizeof(struct ext2_db_entry) * dblist->size;
 	dblist->count = count;
-	dblist->list = malloc(len);
-	if (dblist->list == NULL) {
-		retval = ENOMEM;
+	retval = ext2fs_get_mem(len, (void **) &dblist->list);
+	if (retval)
 		goto cleanup;
-	}
+	
 	if (list)
 		memcpy(dblist->list, list, len);
 	else
@@ -93,7 +103,7 @@ static errcode_t make_dblist(ext2_filsys fs, ino_t size, ino_t count,
 	return 0;
 cleanup:
 	if (dblist)
-		free(dblist);
+		ext2fs_free_mem((void **) &dblist);
 	return retval;
 }
 
@@ -148,24 +158,27 @@ errcode_t ext2fs_copy_dblist(ext2_dblist src, ext2_dblist *dest)
 errcode_t ext2fs_add_dir_block(ext2_dblist dblist, ino_t ino, blk_t blk,
 			       int blockcnt)
 {
-	struct ext2_db_entry 	*nlist, *new;
+	struct ext2_db_entry 	*new_entry;
+	errcode_t		retval;
+	unsigned long		old_size;
 	
 	EXT2_CHECK_MAGIC(dblist, EXT2_ET_MAGIC_DBLIST);
 
 	if (dblist->count >= dblist->size) {
+		old_size = dblist->size * sizeof(struct ext2_db_entry);
 		dblist->size += 100;
-		nlist = realloc(dblist->list,
-				dblist->size * sizeof(struct ext2_db_entry));
-		if (nlist == 0) {
+		retval = ext2fs_resize_mem(old_size, (size_t) dblist->size *
+					   sizeof(struct ext2_db_entry),
+					   (void **) &dblist->list);
+		if (retval) {
 			dblist->size -= 100;
-			return ENOMEM;
+			return retval;
 		}
-		dblist->list = nlist;
 	}
-	new = dblist->list + dblist->count++;
-	new->blk = blk;
-	new->ino = ino;
-	new->blockcnt = blockcnt;
+	new_entry = dblist->list + ( (int) dblist->count++);
+	new_entry->blk = blk;
+	new_entry->ino = ino;
+	new_entry->blockcnt = blockcnt;
 
 	dblist->sorted = 0;
 
@@ -178,7 +191,6 @@ errcode_t ext2fs_add_dir_block(ext2_dblist dblist, ino_t ino, blk_t blk,
 errcode_t ext2fs_set_dir_block(ext2_dblist dblist, ino_t ino, blk_t blk,
 			       int blockcnt)
 {
-	struct ext2_db_entry 	*ent;
 	int			i;
 	
 	EXT2_CHECK_MAGIC(dblist, EXT2_ET_MAGIC_DBLIST);
@@ -191,7 +203,7 @@ errcode_t ext2fs_set_dir_block(ext2_dblist dblist, ino_t ino, blk_t blk,
 		dblist->sorted = 0;
 		return 0;
 	}
-	return ENOENT;
+	return EXT2_ET_DB_NOT_FOUND;
 }
 
 /*
@@ -200,8 +212,8 @@ errcode_t ext2fs_set_dir_block(ext2_dblist dblist, ino_t ino, blk_t blk,
 errcode_t ext2fs_dblist_iterate(ext2_dblist dblist,
 				int (*func)(ext2_filsys fs,
 					    struct ext2_db_entry *db_info,
-					    void	*private),
-				void *private)
+					    void	*priv_data),
+				void *priv_data)
 {
 	ino_t	i;
 	int	ret;
@@ -209,12 +221,12 @@ errcode_t ext2fs_dblist_iterate(ext2_dblist dblist,
 	EXT2_CHECK_MAGIC(dblist, EXT2_ET_MAGIC_DBLIST);
 
 	if (!dblist->sorted) {
-		qsort(dblist->list, dblist->count,
+		qsort(dblist->list, (size_t) dblist->count,
 		      sizeof(struct ext2_db_entry), dir_block_cmp);
 		dblist->sorted = 1;
 	}
 	for (i=0; i < dblist->count; i++) {
-		ret = (*func)(dblist->fs, &dblist->list[i], private);
+		ret = (*func)(dblist->fs, &dblist->list[(int)i], priv_data);
 		if (ret & DBLIST_ABORT)
 			return 0;
 	}
@@ -222,7 +234,7 @@ errcode_t ext2fs_dblist_iterate(ext2_dblist dblist,
 }
 
 
-static int dir_block_cmp(const void *a, const void *b)
+static EXT2_QSORT_TYPE dir_block_cmp(const void *a, const void *b)
 {
 	const struct ext2_db_entry *db_a =
 		(const struct ext2_db_entry *) a;
@@ -230,12 +242,15 @@ static int dir_block_cmp(const void *a, const void *b)
 		(const struct ext2_db_entry *) b;
 
 	if (db_a->blk != db_b->blk)
-		return (db_a->blk - db_b->blk);
+		return (int) (db_a->blk - db_b->blk);
 	
 	if (db_a->ino != db_b->ino)
-		return (db_a->ino - db_b->ino);
+		return (int) (db_a->ino - db_b->ino);
 
-	return (db_a->blockcnt - db_b->blockcnt);
+	return (int) (db_a->blockcnt - db_b->blockcnt);
 }
 
-			     
+int ext2fs_dblist_count(ext2_dblist dblist)
+{
+	return (int) dblist->count;
+}
