@@ -11,13 +11,18 @@
 
 #include <stdio.h>
 #include <string.h>
+#if HAVE_UNISTD_H
 #include <unistd.h>
-#include <stdlib.h>
+#endif
 #if HAVE_ERRNO_H
 #include <errno.h>
 #endif
 
+#if EXT2_FLAT_INCLUDES
+#include "ext2_fs.h"
+#else
 #include <linux/ext2_fs.h>
+#endif
 
 #include "ext2fsP.h"
 
@@ -29,8 +34,8 @@ errcode_t ext2fs_dir_iterate(ext2_filsys fs,
 					 int	offset,
 					 int	blocksize,
 					 char	*buf,
-					 void	*private),
-			     void *private)
+					 void	*priv_data),
+			     void *priv_data)
 {
 	struct		dir_context	ctx;
 	errcode_t	retval;
@@ -46,18 +51,18 @@ errcode_t ext2fs_dir_iterate(ext2_filsys fs,
 	if (block_buf)
 		ctx.buf = block_buf;
 	else {
-		ctx.buf = malloc(fs->blocksize);
-		if (!ctx.buf)
-			return ENOMEM;
+		retval = ext2fs_get_mem(fs->blocksize, (void **) &ctx.buf);
+		if (retval)
+			return retval;
 	}
 	ctx.func = func;
 	ctx.func2 = 0;
-	ctx.private = private;
+	ctx.priv_data = priv_data;
 	ctx.errcode = 0;
-	retval = ext2fs_block_iterate(fs, dir, 0, 0,
-				      ext2fs_process_dir_block, &ctx);
+	retval = ext2fs_block_iterate2(fs, dir, 0, 0,
+				       ext2fs_process_dir_block, &ctx);
 	if (!block_buf)
-		free(ctx.buf);
+		ext2fs_free_mem((void **) &ctx.buf);
 	if (retval)
 		return retval;
 	return ctx.errcode;
@@ -69,10 +74,12 @@ errcode_t ext2fs_dir_iterate(ext2_filsys fs,
  */
 extern int ext2fs_process_dir_block(ext2_filsys  	fs,
 				    blk_t		*blocknr,
-				    int		blockcnt,
-				    void		*private)
+				    e2_blkcnt_t		blockcnt,
+				    blk_t		ref_block,
+				    int			ref_offset,
+				    void		*priv_data)
 {
-	struct dir_context *ctx = (struct dir_context *) private;
+	struct dir_context *ctx = (struct dir_context *) priv_data;
 	int		offset = 0;
 	int		ret = 0;
 	int		changed = 0;
@@ -83,12 +90,12 @@ extern int ext2fs_process_dir_block(ext2_filsys  	fs,
 	if (blockcnt < 0)
 		return 0;
 
+	entry = blockcnt ? DIRENT_OTHER_FILE : DIRENT_DOT_FILE;
+	
 	ctx->errcode = ext2fs_read_dir_block(fs, *blocknr, ctx->buf);
 	if (ctx->errcode)
 		return BLOCK_ABORT;
 
-	entry = blockcnt ? DIRENT_OTHER_FILE : DIRENT_DOT_FILE;
-	
 	while (offset < fs->blocksize) {
 		dirent = (struct ext2_dir_entry *) (ctx->buf + offset);
 		if (!dirent->inode &&
@@ -97,11 +104,11 @@ extern int ext2fs_process_dir_block(ext2_filsys  	fs,
 
 		if (ctx->func)
 			ret = (ctx->func)(dirent, offset, fs->blocksize,
-					  ctx->buf, ctx->private);
+					  ctx->buf, ctx->priv_data);
 		else if (ctx->func2) {
 			ret = (ctx->func2)(ctx->dir, entry, dirent, offset,
 					   fs->blocksize, ctx->buf,
-					   ctx->private);
+					   ctx->priv_data);
 			if (entry < DIRENT_OTHER_FILE)
 				entry++;
 		}
@@ -115,7 +122,7 @@ extern int ext2fs_process_dir_block(ext2_filsys  	fs,
 next:		
 		if (((offset + dirent->rec_len) > fs->blocksize) ||
 		    (dirent->rec_len < 8) ||
-		    ((dirent->name_len+8) > dirent->rec_len)) {
+		    (((dirent->name_len & 0xFF)+8) > dirent->rec_len)) {
 			ctx->errcode = EXT2_ET_DIR_CORRUPTED;
 			return BLOCK_ABORT;
 		}
