@@ -11,13 +11,15 @@
 
 #include <stdio.h>
 #include <string.h>
+#if HAVE_UNISTD_H
 #include <unistd.h>
-#include <stdlib.h>
-#if HAVE_ERRNO_H
-#include <errno.h>
 #endif
 
+#if EXT2_FLAT_INCLUDES
+#include "ext2_fs.h"
+#else
 #include <linux/ext2_fs.h>
+#endif
 
 #include "ext2fs.h"
 
@@ -26,12 +28,14 @@ struct expand_dir_struct {
 	errcode_t	err;
 };
 
-static int expand_dir_proc(ext2_filsys fs,
-			   blk_t	*blocknr,
-			   int	blockcnt,
-			   void	*private)
+static int expand_dir_proc(ext2_filsys		fs,
+			   blk_t		*blocknr,
+			   e2_blkcnt_t		blockcnt,
+			   blk_t		ref_block,
+			   int			ref_offset,
+			   void			*priv_data)
 {
-	struct expand_dir_struct *es = (struct expand_dir_struct *) private;
+	struct expand_dir_struct *es = (struct expand_dir_struct *) priv_data;
 	blk_t	new_blk;
 	static blk_t	last_blk = 0;
 	char		*block;
@@ -54,20 +58,21 @@ static int expand_dir_proc(ext2_filsys fs,
 			return BLOCK_ABORT;
 		}
 		es->done = 1;
+		retval = ext2fs_write_dir_block(fs, new_blk, block);
 	} else {
-		block = malloc(fs->blocksize);
-		if (!block) {
-			es->err = ENOMEM;
+		retval = ext2fs_get_mem(fs->blocksize, (void **) &block);
+		if (retval) {
+			es->err = retval;
 			return BLOCK_ABORT;
 		}
 		memset(block, 0, fs->blocksize);
+		retval = io_channel_write_blk(fs->io, new_blk, 1, block);
 	}	
-	retval = ext2fs_write_dir_block(fs, new_blk, block);
 	if (retval) {
 		es->err = retval;
 		return BLOCK_ABORT;
 	}
-	free(block);
+	ext2fs_free_mem((void **) &block);
 	*blocknr = new_blk;
 	ext2fs_mark_block_bitmap(fs->block_map, new_blk);
 	ext2fs_mark_bb_dirty(fs);
@@ -92,6 +97,9 @@ errcode_t ext2fs_expand_dir(ext2_filsys fs, ino_t dir)
 	if (!(fs->flags & EXT2_FLAG_RW))
 		return EXT2_ET_RO_FILSYS;
 
+	if (!fs->block_map)
+		return EXT2_ET_NO_BLOCK_BITMAP;
+
 	retval = ext2fs_check_directory(fs, dir);
 	if (retval)
 		return retval;
@@ -99,8 +107,8 @@ errcode_t ext2fs_expand_dir(ext2_filsys fs, ino_t dir)
 	es.done = 0;
 	es.err = 0;
 	
-	retval = ext2fs_block_iterate(fs, dir, BLOCK_FLAG_APPEND,
-				      0, expand_dir_proc, &es);
+	retval = ext2fs_block_iterate2(fs, dir, BLOCK_FLAG_APPEND,
+				       0, expand_dir_proc, &es);
 
 	if (es.err)
 		return es.err;
