@@ -12,17 +12,23 @@
 
 #include <stdio.h>
 #include <string.h>
+#if HAVE_UNISTD_H
 #include <unistd.h>
-#include <stdlib.h>
+#endif
 #include <fcntl.h>
 #include <time.h>
+#if HAVE_SYS_STAT_H
 #include <sys/stat.h>
+#endif
+#if HAVE_SYS_TYPES_H
 #include <sys/types.h>
-#if HAVE_ERRNO_H
-#include <errno.h>
 #endif
 
+#if EXT2_FLAT_INCLUDES
+#include "ext2_fs.h"
+#else
 #include <linux/ext2_fs.h>
+#endif
 
 #include "ext2fs.h"
 
@@ -49,7 +55,7 @@
 #ifdef EXT2_DFL_CHECKINTERVAL
 #undef EXT2_DFL_CHECKINTERVAL
 #endif
-#define EXT2_DFL_CHECKINTERVAL (86400 * 180)
+#define EXT2_DFL_CHECKINTERVAL (86400L * 180L)
 
 errcode_t ext2fs_initialize(const char *name, int flags,
 			    struct ext2_super_block *param,
@@ -58,21 +64,21 @@ errcode_t ext2fs_initialize(const char *name, int flags,
 	ext2_filsys	fs;
 	errcode_t	retval;
 	struct ext2_super_block *super;
-	struct ext2fs_sb	*s;
 	int		frags_per_block;
 	int		rem;
 	int		overhead = 0;
 	blk_t		group_block;
 	int		i, j;
-	int		numblocks;
+	blk_t		numblocks;
 	char		*buf;
 
 	if (!param || !param->s_blocks_count)
-		return EINVAL;
+		return EXT2_ET_INVALID_ARGUMENT;
 	
-	fs = (ext2_filsys) malloc(sizeof(struct struct_ext2_filsys));
-	if (!fs)
-		return ENOMEM;
+	retval = ext2fs_get_mem(sizeof(struct struct_ext2_filsys),
+				(void **) &fs);
+	if (retval)
+		return retval;
 	
 	memset(fs, 0, sizeof(struct struct_ext2_filsys));
 	fs->magic = EXT2_ET_MAGIC_EXT2FS_FILSYS;
@@ -80,24 +86,21 @@ errcode_t ext2fs_initialize(const char *name, int flags,
 	retval = manager->open(name, IO_FLAG_RW, &fs->io);
 	if (retval)
 		goto cleanup;
-	fs->device_name = malloc(strlen(name)+1);
-	if (!fs->device_name) {
-		retval = ENOMEM;
+	fs->io->app_data = fs;
+	retval = ext2fs_get_mem(strlen(name)+1, (void **) &fs->device_name);
+	if (retval)
 		goto cleanup;
-	}
+
 	strcpy(fs->device_name, name);
-	fs->super = super = malloc(SUPERBLOCK_SIZE);
-	if (!super) {
-		retval = ENOMEM;
+	retval = ext2fs_get_mem(SUPERBLOCK_SIZE, (void **) &super);
+	if (retval)
 		goto cleanup;
-	}
+	fs->super = super;
+
 	memset(super, 0, SUPERBLOCK_SIZE);
-	s = (struct ext2fs_sb *) super;
 
 #define set_field(field, default) (super->field = param->field ? \
 				   param->field : (default))
-#define set_ext2_field(field, default) (s->field = param->field ? \
-					param->field : (default))
 
 	super->s_magic = EXT2_SUPER_MAGIC;
 	super->s_state = EXT2_VALID_FS;
@@ -107,15 +110,15 @@ errcode_t ext2fs_initialize(const char *name, int flags,
 	set_field(s_first_data_block, super->s_log_block_size ? 0 : 1);
 	set_field(s_max_mnt_count, EXT2_DFL_MAX_MNT_COUNT);
 	set_field(s_errors, EXT2_ERRORS_DEFAULT);
-	set_ext2_field(s_feature_compat, 0);
-	set_ext2_field(s_feature_incompat, 0);
-	set_ext2_field(s_feature_ro_compat, 0);
-	if (s->s_feature_incompat & ~EXT2_LIB_FEATURE_INCOMPAT_SUPP)
+#ifdef EXT2_DYNAMIC_REV
+	set_field(s_feature_compat, 0);
+	set_field(s_feature_incompat, 0);
+	set_field(s_feature_ro_compat, 0);
+	if (super->s_feature_incompat & ~EXT2_LIB_FEATURE_INCOMPAT_SUPP)
 		return EXT2_ET_UNSUPP_FEATURE;
-	if (s->s_feature_ro_compat & ~EXT2_LIB_FEATURE_RO_COMPAT_SUPP)
+	if (super->s_feature_ro_compat & ~EXT2_LIB_FEATURE_RO_COMPAT_SUPP)
 		return EXT2_ET_RO_UNSUPP_FEATURE;
 
-#ifdef EXT2_DYNAMIC_REV
 	set_field(s_rev_level, EXT2_GOOD_OLD_REV);
 	if (super->s_rev_level >= EXT2_DYNAMIC_REV) {
 		set_field(s_first_ino, EXT2_GOOD_OLD_FIRST_INO);
@@ -139,7 +142,7 @@ errcode_t ext2fs_initialize(const char *name, int flags,
 	super->s_blocks_count = param->s_blocks_count;
 	super->s_r_blocks_count = param->s_r_blocks_count;
 	if (super->s_r_blocks_count >= param->s_blocks_count) {
-		retval = EINVAL;
+		retval = EXT2_ET_INVALID_ARGUMENT;
 		goto cleanup;
 	}
 
@@ -154,7 +157,8 @@ retry:
 			   EXT2_DESC_PER_BLOCK(super) - 1)
 		/ EXT2_DESC_PER_BLOCK(super);
 
-	set_field(s_inodes_count, (super->s_blocks_count*fs->blocksize)/4096);
+	/* n.b., fs->blocksize is <= 4096 */
+	set_field(s_inodes_count, super->s_blocks_count/(4096/fs->blocksize));
 
 	/*
 	 * There should be at least as many inodes as the user
@@ -198,13 +202,24 @@ retry:
 		fs->group_desc_count;
 	super->s_free_inodes_count = super->s_inodes_count;
 
+	/*
+	 * Overhead is the number of bookkeeping blocks per group.  It
+	 * includes the superblock backup, the group descriptor
+	 * backups, the inode bitmap, the block bitmap, and the inode
+	 * table.
+	 *
+	 * XXX Not all block groups need the descriptor blocks, but
+	 * being clever is tricky...
+	 */
+	overhead = (int) (3 + fs->desc_blocks + fs->inode_blocks_per_group);
+	
 	/*
 	 * See if the last group is big enough to support the
 	 * necessary data structures.  If not, we need to get rid of
 	 * it.
 	 */
-	rem = (super->s_blocks_count - super->s_first_data_block) %
-		super->s_blocks_per_group;
+	rem = (int) ((super->s_blocks_count - super->s_first_data_block) %
+		     super->s_blocks_per_group);
 	if ((fs->group_desc_count == 1) && rem && (rem < overhead))
 		return EXT2_ET_TOOSMALL;
 	if (rem && (rem < overhead+50)) {
@@ -218,11 +233,10 @@ retry:
 	 * count.
 	 */
 
-	buf = malloc(strlen(fs->device_name) + 80);
-	if (!buf) {
-		retval = ENOMEM;
+	retval = ext2fs_get_mem(strlen(fs->device_name) + 80,
+				(void **) &buf);
+	if (retval)
 		goto cleanup;
-	}
 	
 	sprintf(buf, "block bitmap for %s", fs->device_name);
 	retval = ext2fs_allocate_block_bitmap(fs, buf, &fs->block_map);
@@ -234,14 +248,14 @@ retry:
 	if (retval)
 		goto cleanup;
 
-	free(buf);
+	ext2fs_free_mem((void **) &buf);
 
-	fs->group_desc = malloc(fs->desc_blocks * fs->blocksize);
-	if (!fs->group_desc) {
-		retval = ENOMEM;
+	retval = ext2fs_get_mem((size_t) fs->desc_blocks * fs->blocksize,
+				(void **) &fs->group_desc);
+	if (retval)
 		goto cleanup;
-	}
-	memset(fs->group_desc, 0, fs->desc_blocks * fs->blocksize);
+
+	memset(fs->group_desc, 0, (size_t) fs->desc_blocks * fs->blocksize);
 
 	/*
 	 * Reserve the superblock and group descriptors for each
