@@ -11,20 +11,22 @@
 
 #include <stdio.h>
 #include <string.h>
+#if HAVE_UNISTD_H
 #include <unistd.h>
-#include <stdlib.h>
-#if HAVE_ERRNO_H
-#include <errno.h>
 #endif
 
 /* #define NAMEI_DEBUG */
 
+#if EXT2_FLAT_INCLUDES
+#include "ext2_fs.h"
+#else
 #include <linux/ext2_fs.h>
+#endif
 
 #include "ext2fs.h"
 
 static errcode_t open_namei(ext2_filsys fs, ino_t root, ino_t base,
-			    const char *pathname, int pathlen, int follow,
+			    const char *pathname, size_t pathlen, int follow,
 			    int link_count, char *buf, ino_t *res_inode);
 
 static errcode_t follow_link(ext2_filsys fs, ino_t root, ino_t dir,
@@ -51,12 +53,12 @@ static errcode_t follow_link(ext2_filsys fs, ino_t root, ino_t dir,
 		return EXT2_ET_SYMLINK_LOOP;
 	}
 	if (ei.i_blocks) {
-		buffer = malloc (fs->blocksize);
-		if (!buffer)
-			return ENOMEM;
+		retval = ext2fs_get_mem(fs->blocksize, (void **) &buffer);
+		if (retval)
+			return retval;
 		retval = io_channel_read_blk(fs->io, ei.i_block[0], 1, buffer);
 		if (retval) {
-			free(buffer);
+			ext2fs_free_mem((void **) &buffer);
 			return retval;
 		}
 		pathname = buffer;
@@ -65,7 +67,7 @@ static errcode_t follow_link(ext2_filsys fs, ino_t root, ino_t dir,
 	retval = open_namei(fs, root, dir, pathname, ei.i_size, 1,
 			    link_count, buf, res_inode);
 	if (buffer)
-		free (buffer);
+		ext2fs_free_mem((void **) &buffer);
 	return retval;
 }
 
@@ -113,7 +115,7 @@ static errcode_t dir_namei(ext2_filsys fs, ino_t root, ino_t dir,
 }
 
 static errcode_t open_namei(ext2_filsys fs, ino_t root, ino_t base,
-			    const char *pathname, int pathlen, int follow,
+			    const char *pathname, size_t pathlen, int follow,
 			    int link_count, char *buf, ino_t *res_inode)
 {
 	const char *basename;
@@ -157,14 +159,14 @@ errcode_t ext2fs_namei(ext2_filsys fs, ino_t root, ino_t cwd,
 	
 	EXT2_CHECK_MAGIC(fs, EXT2_ET_MAGIC_EXT2FS_FILSYS);
 
-	buf = malloc(fs->blocksize);
-	if (!buf)
-		return ENOMEM;
+	retval = ext2fs_get_mem(fs->blocksize, (void **) &buf);
+	if (retval)
+		return retval;
 	
 	retval = open_namei(fs, root, cwd, name, strlen(name), 0, 0,
 			    buf, inode);
 
-	free(buf);
+	ext2fs_free_mem((void **) &buf);
 	return retval;
 }
 
@@ -176,14 +178,14 @@ errcode_t ext2fs_namei_follow(ext2_filsys fs, ino_t root, ino_t cwd,
 	
 	EXT2_CHECK_MAGIC(fs, EXT2_ET_MAGIC_EXT2FS_FILSYS);
 
-	buf = malloc(fs->blocksize);
-	if (!buf)
-		return ENOMEM;
+	retval = ext2fs_get_mem(fs->blocksize, (void **) &buf);
+	if (retval)
+		return retval;
 	
 	retval = open_namei(fs, root, cwd, name, strlen(name), 1, 0,
 			    buf, inode);
 
-	free(buf);
+	ext2fs_free_mem((void **) &buf);
 	return retval;
 }
 
@@ -195,13 +197,13 @@ extern errcode_t ext2fs_follow_link(ext2_filsys fs, ino_t root, ino_t cwd,
 	
 	EXT2_CHECK_MAGIC(fs, EXT2_ET_MAGIC_EXT2FS_FILSYS);
 
-	buf = malloc(fs->blocksize);
-	if (!buf)
-		return ENOMEM;
-	
+	retval = ext2fs_get_mem(fs->blocksize, (void **) &buf);
+	if (retval)
+		return retval;
+
 	retval = follow_link(fs, root, cwd, inode, 0, buf, res_inode);
 
-	free(buf);
+	ext2fs_free_mem((void **) &buf);
 	return retval;
 }
 
