@@ -14,18 +14,20 @@
 #if HAVE_UNISTD_H
 #include <unistd.h>
 #endif
-#if HAVE_STDLIB_H
-#include <stdlib.h>
-#endif
 #include <fcntl.h>
 #include <time.h>
-#if HAVE_ERRNO_H
-#include <errno.h>
-#endif
+#if HAVE_SYS_STAT_H
 #include <sys/stat.h>
+#endif
+#if HAVE_SYS_TYPES_H
 #include <sys/types.h>
+#endif
 
+#if EXT2_FLAT_INCLUDES
+#include "ext2_fs.h"
+#else
 #include <linux/ext2_fs.h>
+#endif
 
 #include "ext2fs.h"
 
@@ -44,16 +46,18 @@ errcode_t ext2fs_open(const char *name, int flags, int superblock,
 {
 	ext2_filsys	fs;
 	errcode_t	retval;
-	int		i, j, group_block, groups_per_block;
+	int		i, j, groups_per_block;
+	blk_t		group_block;
 	char		*dest;
 	struct ext2_group_desc *gdp;
 	struct ext2fs_sb	*s;
 	
 	EXT2_CHECK_MAGIC(manager, EXT2_ET_MAGIC_IO_MANAGER);
-	
-	fs = (ext2_filsys) malloc(sizeof(struct struct_ext2_filsys));
-	if (!fs)
-		return ENOMEM;
+
+	retval = ext2fs_get_mem(sizeof(struct struct_ext2_filsys),
+				(void **) &fs);
+	if (retval)
+		return retval;
 	
 	memset(fs, 0, sizeof(struct struct_ext2_filsys));
 	fs->magic = EXT2_ET_MAGIC_EXT2FS_FILSYS;
@@ -62,17 +66,14 @@ errcode_t ext2fs_open(const char *name, int flags, int superblock,
 			       &fs->io);
 	if (retval)
 		goto cleanup;
-	fs->device_name = malloc(strlen(name)+1);
-	if (!fs->device_name) {
-		retval = ENOMEM;
+	fs->io->app_data = fs;
+	retval = ext2fs_get_mem(strlen(name)+1, (void **) &fs->device_name);
+	if (retval)
 		goto cleanup;
-	}
 	strcpy(fs->device_name, name);
-	fs->super = malloc(SUPERBLOCK_SIZE);
-	if (!fs->super) {
-		retval = ENOMEM;
+	retval = ext2fs_get_mem(SUPERBLOCK_SIZE, (void **) &fs->super);
+	if (retval)
 		goto cleanup;
-	}
 
 	/*
 	 * If the user specifies a specific block # for the
@@ -82,7 +83,7 @@ errcode_t ext2fs_open(const char *name, int flags, int superblock,
 	 */
 	if (superblock) {
 		if (!block_size) {
-			retval = EINVAL;
+			retval = EXT2_ET_INVALID_ARGUMENT;
 			goto cleanup;
 		}
 		io_channel_set_blksize(fs->io, block_size);
@@ -169,11 +170,10 @@ errcode_t ext2fs_open(const char *name, int flags, int superblock,
 	fs->desc_blocks = (fs->group_desc_count +
 			   EXT2_DESC_PER_BLOCK(fs->super) - 1)
 		/ EXT2_DESC_PER_BLOCK(fs->super);
-	fs->group_desc = malloc(fs->desc_blocks * fs->blocksize);
-	if (!fs->group_desc) {
-		retval = ENOMEM;
+	retval = ext2fs_get_mem(fs->desc_blocks * fs->blocksize,
+				(void **) &fs->group_desc);
+	if (retval)
 		goto cleanup;
-	}
 	if (!group_block)
 		group_block = fs->super->s_first_data_block + 1;
 	dest = (char *) fs->group_desc;
