@@ -11,14 +11,23 @@
 
 #include <stdio.h>
 #include <string.h>
+#if HAVE_UNISTD_H
 #include <unistd.h>
-#include <stdlib.h>
+#endif
 #include <fcntl.h>
 #include <time.h>
+#if HAVE_SYS_STAT_H
 #include <sys/stat.h>
+#endif
+#if HAVE_SYS_TYPES_H
 #include <sys/types.h>
+#endif
 
+#if EXT2_FLAT_INCLUDES
+#include "ext2_fs.h"
+#else
 #include <linux/ext2_fs.h>
+#endif
 
 #include "ext2fs.h"
 
@@ -30,14 +39,22 @@ struct read_bb_record {
 /*
  * Helper function for ext2fs_read_bb_inode()
  */
+#ifdef __TURBOC__
+#pragma argsused
+#endif
 static int mark_bad_block(ext2_filsys fs, blk_t *block_nr,
-			     int blockcnt, void *private)
+			  e2_blkcnt_t blockcnt, blk_t ref_block,
+			  int ref_offset, void *priv_data)
 {
-	struct read_bb_record *rb = (struct read_bb_record *) private;
+	struct read_bb_record *rb = (struct read_bb_record *) priv_data;
 	
 	if (blockcnt < 0)
 		return 0;
 	
+	if ((*block_nr < fs->super->s_first_data_block) ||
+	    (*block_nr >= fs->super->s_blocks_count))
+		return 0;	/* Ignore illegal blocks */
+
 	rb->err = ext2fs_badblocks_list_add(rb->bb_list, *block_nr);
 	if (rb->err)
 		return BLOCK_ABORT;
@@ -52,7 +69,7 @@ errcode_t ext2fs_read_bb_inode(ext2_filsys fs, ext2_badblocks_list *bb_list)
 	errcode_t	retval;
 	struct read_bb_record rb;
 	struct ext2_inode inode;
-	int	numblocks;
+	blk_t	numblocks;
 
 	EXT2_CHECK_MAGIC(fs, EXT2_ET_MAGIC_EXT2FS_FILSYS);
 
@@ -60,7 +77,11 @@ errcode_t ext2fs_read_bb_inode(ext2_filsys fs, ext2_badblocks_list *bb_list)
 		retval = ext2fs_read_inode(fs, EXT2_BAD_INO, &inode);
 		if (retval)
 			return retval;
-		numblocks = (inode.i_blocks / (fs->blocksize / 512)) + 20;
+		if (inode.i_blocks < 500)
+			numblocks = (inode.i_blocks /
+				     (fs->blocksize / 512)) + 20;
+		else
+			numblocks = 500;
 		retval = ext2fs_badblocks_list_create(bb_list, numblocks);
 		if (retval)
 			return retval;
@@ -68,7 +89,7 @@ errcode_t ext2fs_read_bb_inode(ext2_filsys fs, ext2_badblocks_list *bb_list)
 
 	rb.bb_list = *bb_list;
 	rb.err = 0;
-	retval = ext2fs_block_iterate(fs, EXT2_BAD_INO, 0, 0,
+	retval = ext2fs_block_iterate2(fs, EXT2_BAD_INO, 0, 0,
 				      mark_bad_block, &rb);
 	if (retval)
 		return retval;
