@@ -9,17 +9,19 @@
  * %End-Header%
  */
 
-#include <et/com_err.h>
+#if HAVE_UNISTD_H
 #include <unistd.h>
-#include <stdlib.h>
+#endif
 #include <string.h>
 #include <stdio.h>
 #include <ctype.h>
-#ifdef HAVE_ERRNO_H
-#include <errno.h>
+
+#if EXT2_FLAT_INCLUDES
+#include "ext2_fs.h"
+#else
+#include <linux/ext2_fs.h>
 #endif
 
-#include <linux/ext2_fs.h>
 #include "ext2fs.h"
 
 #include "../../version.h"
@@ -32,7 +34,7 @@ int ext2fs_parse_version_string(const char *ver_string)
 	const char *cp;
 	int version = 0;
 
-	for (cp = lib_version; *cp; cp++) {
+	for (cp = ver_string; *cp; cp++) {
 		if (!isdigit(*cp))
 			continue;
 		version = (version * 10) + (*cp - '0');
