@@ -109,7 +109,7 @@ static int get_node_id(unsigned char *node_id)
 	}
 	n = ifc.ifc_len;
 	for (i = 0; i < n; i+= ifreq_size(*ifr) ) {
-		ifrp = (struct ifreq *)((caddr_t) ifc.ifc_buf+i);
+		ifrp = (struct ifreq *)((char *) ifc.ifc_buf+i);
 		strncpy(ifr.ifr_name, ifrp->ifr_name, IFNAMSIZ);
 #ifdef SIOCGIFHWADDR
 		if (ioctl(sd, SIOCGIFHWADDR, &ifr) < 0)
@@ -151,7 +151,7 @@ static int get_clock(__u32 *clock_high, __u32 *clock_low, __u16 *ret_clock_seq)
 	static struct timeval		last = {0, 0};
 	static __u16			clock_seq;
 	struct timeval 			tv;
-	unsigned long long		clock;
+	unsigned long long		clock_reg;
 	
 try_again:
 	gettimeofday(&tv, 0);
@@ -174,12 +174,12 @@ try_again:
 	} else
 		adjustment = 0;
 	
-	clock = tv.tv_usec*10 + adjustment;
-	clock += ((unsigned long long) tv.tv_sec)*10000000;
-	clock += (((unsigned long long) 0x01B21DD2) << 32) + 0x13814000;
+	clock_reg = tv.tv_usec*10 + adjustment;
+	clock_reg += ((unsigned long long) tv.tv_sec)*10000000;
+	clock_reg += (((unsigned long long) 0x01B21DD2) << 32) + 0x13814000;
 
-	*clock_high = clock >> 32;
-	*clock_low = clock;
+	*clock_high = clock_reg >> 32;
+	*clock_low = clock_reg;
 	*ret_clock_seq = clock_seq;
 	return 0;
 }
