@@ -19,9 +19,6 @@
 #include <sys/types.h>
 #include <dirent.h>
 #include <fcntl.h>
-#ifdef HAVE_GETOPT_H
-#include <getopt.h>
-#endif
 #include <stdio.h>
 #include <stdlib.h>
 #include <unistd.h>
@@ -54,13 +51,13 @@ unsigned long af;
 unsigned long rf;
 unsigned long sf;
 
-static void volatile fatal_error (const char * fmt_string, int errcode)
+static void fatal_error(const char * fmt_string, int errcode)
 {
 	fprintf (stderr, fmt_string, program_name);
 	exit (errcode);
 }
 
-#define usage() fatal_error ("usage: %s [-RV] [-+=AacdisSu] [-v version] files...\n", \
+#define usage() fatal_error("usage: %s [-RV] [-+=AacdisSu] [-v version] files...\n", \
 			     1)
 
 static int decode_arg (int * i, int argc, char ** argv)
@@ -302,7 +299,7 @@ static int chattr_dir_proc (const char * dir_name, struct dirent * de, void * pr
 	return 0;
 }
 
-void main (int argc, char ** argv)
+int main (int argc, char ** argv)
 {
 	int i, j;
 	int end_arg = 0;
@@ -334,4 +331,5 @@ void main (int argc, char ** argv)
 	}
 	for (j = i; j < argc; j++)
 		change_attributes (argv[j]);
+	exit(0);
 }
